/*======================================================================

    PCMCIA controller probe

    Written by David Hinds, dhinds@allegro.stanford.edu
    
======================================================================*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <asm/io.h>

#include "i82365.h"
#include "pd67xx.h"
#include "tcic.h"

static const char *version =
    "probe.c v$Revision: 1.5 $: $Date: 1994/08/18 04:39:11 $ (David Hinds)\n";

/*====================================================================*/

static u_char i365_get(u_short sock, u_short reg)
{
    u_short port = I365_PORT(sock);
    u_char val = I365_REG(sock, reg);
    outb_p(val, port); val = inb_p(port+1);
    return val;
}

static void i365_set(u_short sock, u_short reg, u_char data)
{
    u_short port = I365_PORT(sock);
    u_char val = I365_REG(sock, reg);
    outb_p(val, port); outb_p(data, port+1);
}

/*====================================================================*/

int i365_probe(void)
{
    int val, id, sock, done;
    char *name = "i82365sl";

    printf("Intel PCIC probe: ");
    
    id = sock = done = 0;
    ioperm(I365_BASE, 4, 1);
    ioperm(0x80, 1, 1);
    for (; sock < 8; sock++) {
	val = i365_get(sock, I365_IDENT);
	switch (val) {
	case 0x82: case 0x83:
	    name = "i82365sl";
	    id = val & 7;
	    break;
	case 0x84:
	    name = "VLSI 82C146";
	    id = 0;
	    break;
	case 0x88: case 0x89:
	    name = "IBM Clone";
	    id = val & 7;
	    break;
	default:
	    done = 1;
	}
	if (done) break;
    }
    
    if (sock == 0) {
	printf("not found.\n");
	return -ENODEV;
    }
    
    /* Check for Cirrus CL-PD67xx chips */
    i365_set(0, PD67_CHIP_INFO, 0);
    val = i365_get(0, PD67_CHIP_INFO);
    if ((val & PD67_INFO_CHIP_ID) == PD67_INFO_CHIP_ID) {
	val = i365_get(0, PD67_CHIP_INFO);
	if ((val & PD67_INFO_CHIP_ID) == 0) {
	    if (val & PD67_INFO_SLOTS)
		name = "Cirrus CL-PD672x";
	    else {
		name = "Cirrus CL-PD6710";
		sock = 1;
	    }
	    id = 8 - ((val & PD67_INFO_REV) >> 2);
	}
    }

    printf("%s rev %d found, %d sockets\n", name, id, sock);
    return 0;
    
} /* i365_probe */
  
/*====================================================================*/

static int tcic_base = TCIC_BASE;

static u_short tcic_getw(u_char reg)
{
    u_short val = inw(tcic_base+reg);
    return val;
}

static void tcic_setw(u_char reg, u_short data)
{
    outw(data, tcic_base+reg);
}

int tcic_probe(void)
{
    int sock;

    printf("Databook TCIC/2 probe: ");
    sock = 0;
    ioperm(tcic_base, 16, 1);
    ioperm(0x80, 1, 1);
    
    tcic_setw(TCIC_ADDR, 0);
    if (tcic_getw(TCIC_ADDR) == 0) {
	tcic_setw(TCIC_ADDR, 0xc3a5);
	if (tcic_getw(TCIC_ADDR) == 0xc3a5) sock = 2;
    }

    if (sock == 0) {
	printf("not found.\n");
	return -ENODEV;
    }
    
    printf("%d sockets\n", sock);

    return 0;
    
} /* tcic_probe */

/*====================================================================*/

void main(int argc, char *argv[])
{
    if ((argc > 1) && (strcmp(argv[1], "-v") == 0))
	printf("%s", version);
    
    i365_probe();
    tcic_probe();
}
