/*
 * In the absence of a suitable I/O and memory resource manager,
 * this will have to do.
 */

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>

#include "iohack.h"

/* Each address registrar is an ordered circular linked list */
static resource_entry_t io_registrar = { 0, 0, NULL, &io_registrar };
static resource_entry_t mem_registrar = { 0, 0, NULL, &mem_registrar };

static resource_entry_t *find_gap(resource_entry_t *root,
				  resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    if (entry->base > entry->base+entry->num-1)
	return NULL;
    save_flags(flags);
    cli();
    for (p = root; ; p = p->next) {
	if ((p != root) && (p->base+p->num-1 >= entry->base)) {
	    restore_flags(flags);
	    return NULL;
	}
	if ((p->next == root) ||
	    (p->next->base > entry->base+entry->num-1))
	    break;
    }
    restore_flags(flags);
    return p;
}

static int insert_link(resource_entry_t *root, resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    save_flags(flags);
    cli();
    p = find_gap(root, entry);
    if (p == NULL) {
	restore_flags(flags);
	return -EBUSY;
    }
    entry->next = p->next;
    p->next = entry;
    restore_flags(flags);
    return 0;
}

static int delete_link(resource_entry_t *root, resource_entry_t *entry)
{
    unsigned long flags;
    resource_entry_t *p;
    
    save_flags(flags);
    cli();
    for (p = root; p->next != entry; p = p->next) {
	if (p->next == root) {
	    restore_flags(flags);
	    return -EINVAL;
	}
    }
    p->next = entry->next;
    restore_flags(flags);
    return 0;
}

int check_io_region(resource_entry_t *io)
{
    return (find_gap(&io_registrar, io) == NULL) ? -EBUSY : 0;
}

int alloc_io_region(resource_entry_t *io)
{
    return insert_link(&io_registrar, io);
}

int free_io_region(resource_entry_t *io)
{
    return delete_link(&io_registrar, io);
}

int check_mem_region(resource_entry_t *mem)
{
    return (find_gap(&mem_registrar, mem) == NULL) ? -EBUSY : 0;
}

int alloc_mem_region(resource_entry_t *mem)
{
    return insert_link(&mem_registrar, mem);
}

int free_mem_region(resource_entry_t *mem)
{
    return delete_link(&mem_registrar, mem);
}

#if 0

int get_port_list(char *buf)
{
    int len = 0;
    resource_entry_t *p;

    for (p = &port_registrar; ; p = p->next) {
	len += sprintf(buf+len, "[%#6lx-%#6lx]: %s\n",
		       p->base, p->base+len, p->dev);
	if (p->next == &port_registrar) break;
    }
    return len;
}

int get_mem_list(char *buf)
{
    int len = 0;
    resource_entry_t *p;

    for (p = &mem_registrar; ; p = p->next) {
	len += sprintf(buf+len, "[%#8lx-%#8lx]: %s\n",
		       p->base, p->base+len, p->dev);
	if (p->next == &mem_registrar) break;
    }
    return len;
}

#endif
